@@ -243,8 +243,8 @@ void Frame_Glom::set_databases_selected(const Glib::ustring& strName)
 
 void Frame_Glom::on_box_tables_selected(const Glib::ustring& strName)
 {
-  if (m_pDialog_Tables)
-    m_pDialog_Tables->hide(); //cause_close();
+  if(m_pDialog_Tables)
+    m_pDialog_Tables->hide();
 
   show_table(strName);
 }
@@ -357,9 +357,10 @@ void Frame_Glom::show_table(const Glib::ustring& table_name, const Gnome::Gda::V
     //Show the table:
     m_table_name = table_name;
     Glib::ustring strMode;
-
+#ifndef GLOM_ENABLE_CLIENT_ONLY
     //Update the document with any new information in the database if necessary (though the database _should never have changed information)
     update_table_in_document_from_database();
+#endif // !GLOM_ENABLE_CLIENT_ONLY
 
     //Update user-level dependent UI:
     if(pApp)
@@ -371,29 +372,42 @@ void Frame_Glom::show_table(const Glib::ustring& table_name, const Gnome::Gda::V
       {
         strMode = _("Data");
         FoundSet found_set;
+
+        //Start with the last-used found set (sort order and where clause)
+        //for this layout:
+        //(This would be ignored anyway if a details primary key is specified.)
+        Document_Glom* document = get_document(); 
+        if(document)
+          found_set = document->get_criteria_current(m_table_name);
+
+        //Make sure that this is set:
         found_set.m_table_name = m_table_name;
 
-        //Sort by the ID, just so we sort by something, so that the order is predictable:
-        sharedptr<Field> field_primary_key = get_field_primary_key_for_table(m_table_name);
-        if(field_primary_key)
+        //If there is no saved sort clause, 
+        //then sort by the ID, just so we sort by something, so that the order is predictable:
+        if(found_set.m_sort_clause.empty())
         {
-          sharedptr<LayoutItem_Field> layout_item_sort = sharedptr<LayoutItem_Field>::create();
-          layout_item_sort->set_full_field_details(field_primary_key);
+          sharedptr<Field> field_primary_key = get_field_primary_key_for_table(m_table_name);
+          if(field_primary_key)
+          {
+            sharedptr<LayoutItem_Field> layout_item_sort = sharedptr<LayoutItem_Field>::create();
+            layout_item_sort->set_full_field_details(field_primary_key);
 
-          found_set.m_sort_clause.clear();
+            found_set.m_sort_clause.clear();
 
-          //Avoid the sort clause if the found set will include too many records, 
-          //because that would be too slow.
-          //The user can explicitly request a sort later, by clicking on a column header.
-          //TODO_Performance: This causes an almost-duplicate COUNT query (we do it in the treemodel too), but it's not that slow. 
-          sharedptr<LayoutItem_Field> layout_item_temp = sharedptr<LayoutItem_Field>::create();
-          layout_item_temp->set_full_field_details(field_primary_key);
-          type_vecLayoutFields layout_fields;
-          layout_fields.push_back(layout_item_temp);
-          const Glib::ustring sql_query_without_sort = Utils::build_sql_select_with_where_clause(found_set.m_table_name, layout_fields, found_set.m_where_clause, found_set.m_extra_join, type_sort_clause(), found_set.m_extra_group_by);
-          const int count = Base_DB::count_rows_returned_by(sql_query_without_sort);
-          if(count < 10000) //Arbitrary large number.
-            found_set.m_sort_clause.push_back( type_pair_sort_field(layout_item_sort, true /* ascending */) );
+            //Avoid the sort clause if the found set will include too many records, 
+            //because that would be too slow.
+            //The user can explicitly request a sort later, by clicking on a column header.
+            //TODO_Performance: This causes an almost-duplicate COUNT query (we do it in the treemodel too), but it's not that slow. 
+            sharedptr<LayoutItem_Field> layout_item_temp = sharedptr<LayoutItem_Field>::create();
+            layout_item_temp->set_full_field_details(field_primary_key);
+            type_vecLayoutFields layout_fields;
+            layout_fields.push_back(layout_item_temp);
+            const Glib::ustring sql_query_without_sort = Utils::build_sql_select_with_where_clause(found_set.m_table_name, layout_fields, found_set.m_where_clause, found_set.m_extra_join, type_sort_clause(), found_set.m_extra_group_by);
+            const int count = Base_DB::count_rows_returned_by(sql_query_without_sort);
+            if(count < 10000) //Arbitrary large number.
+              found_set.m_sort_clause.push_back( type_pair_sort_field(layout_item_sort, true /* ascending */) );
+          }
         }
 
         //Show the wanted records in the notebook, showing details for a particular record if wanted:
@@ -660,7 +674,6 @@ void Frame_Glom::export_data_to_stream(std::ostream& the_stream, const FoundSet&
     }
   }
 }
-#endif // !GLOM_ENABLE_CLIENT_ONLY
 
 void Frame_Glom::on_menu_file_import()
 {
@@ -714,6 +727,7 @@ void Frame_Glom::on_menu_file_import()
     }
   }
 }
+#endif // !GLOM_ENABLE_CLIENT_ONLY
 
 void Frame_Glom::on_menu_file_print()
 {
@@ -727,6 +741,12 @@ void Frame_Glom::on_menu_file_print_edit_layouts()
 {
   on_menu_developer_print_layouts();
 }
+
+void Frame_Glom::show_layout_toolbar (bool show)
+{
+  m_Notebook_Data.show_layout_toolbar(show);
+}
+
 #endif // !GLOM_ENABLE_CLIENT_ONLY
 
 void Frame_Glom::on_menu_Mode_Data()
@@ -1111,6 +1131,7 @@ void Frame_Glom::show_table_title()
   }
 }
 
+#ifndef GLOM_ENABLE_CLIENT_ONLY
 void Frame_Glom::update_table_in_document_from_database()
 {
   //Add any new/changed information from the database to the document
@@ -1158,7 +1179,15 @@ void Frame_Glom::update_table_in_document_from_database()
               //The database has different information. We assume that the information in the database is newer.
 
               //Update the field information:
-              field_info_db->set_auto_increment( field_document->get_auto_increment() ); //libgda does not report it from the database properly.
+
+              // Ignore things that libgda does not report from the database properly:
+              // We do this also in Field::field_info_from_database_is_equal() and 
+              // Base_DB::get_fields_for_table(), 
+              // so make sure that we ignore the same things everywhere always
+              // TODO: Avoid that duplication?
+              field_info_db->set_auto_increment( field_document->get_auto_increment() );
+              field_info_db->set_default_value( field_document->get_default_value() );
+
               (*iterFindDoc)->set_field_info( field_info_db );
 
               document_must_be_updated = true;
@@ -1191,6 +1220,7 @@ void Frame_Glom::update_table_in_document_from_database()
       pDoc->set_table_fields(m_table_name, fieldsActual);
   }
 }
+#endif // !GLOM_ENABLE_CLIENT_ONLY
 
 void Frame_Glom::set_document(Document_Glom* pDocument)
 {
@@ -1970,6 +2000,7 @@ void Frame_Glom::on_menu_print_layout_selected(const Glib::ustring& print_layout
     return;
 
   Canvas_PrintLayout canvas;
+  add_view(&canvas); //So it has access to the document.
   canvas.set_print_layout(m_table_name, print_layout);
 
   //Create a new PrintOperation with our PageSetup and PrintSettings:
@@ -1999,6 +2030,8 @@ void Frame_Glom::on_menu_print_layout_selected(const Glib::ustring& print_layout
     std::cerr << "An error occurred while trying to run a print operation:"
         << ex.what() << std::endl;
   }
+
+  remove_view(&canvas);
 }
 #endif // !GLOM_ENABLE_CLIENT_ONLY
 
