@@ -22,7 +22,7 @@
 #include "dialog_buttonscript.h"
 #include "../python_embed/glom_python.h"
 #include <glom/libglom/data_structure/glomconversions.h>
-#include <gtksourceviewmm/sourcelanguagesmanager.h>
+#include <gtksourceviewmm/sourcelanguagemanager.h>
 
 
 //#include <libgnome/gnome-i18n.h>
@@ -36,22 +36,25 @@ Dialog_ButtonScript::Dialog_ButtonScript(BaseObjectType* cobject, const Glib::Re
 {
   refGlade->get_widget("textview_calculation",  m_text_view);
   refGlade->get_widget("button_test",  m_button_test);
+  refGlade->get_widget("entry_title",  m_entry_title);
 
   m_button_test->signal_clicked().connect( sigc::mem_fun(*this, &Dialog_ButtonScript::on_button_test) );
+
+  // Set a monospace font
+  m_text_view->modify_font(Pango::FontDescription("Monospace"));
   //on_foreach_connect(*this);
 
   //Dialog_Properties::set_modified(false);
 
   //Set the SourceView to do syntax highlighting for Python:
-  //TODO: Shouldn't this be a singleton?
-  Glib::RefPtr<gtksourceview::SourceLanguagesManager> languages_manager = gtksourceview::SourceLanguagesManager::create();
+  Glib::RefPtr<gtksourceview::SourceLanguageManager> languages_manager = gtksourceview::SourceLanguageManager::get_default();
 
-  Glib::RefPtr<gtksourceview::SourceLanguage> language = languages_manager->get_language_from_mime_type("application/x-python");
+  Glib::RefPtr<gtksourceview::SourceLanguage> language = languages_manager->get_language("python"); //This is the GtkSourceView language ID.
   if(language)
   {
-     //Createa a new buffer and set it, instead of getting the default buffer, in case libglade has tried to set it, using the wrong buffer type:
+     //Create a new buffer and set it, instead of getting the default buffer, in case libglade has tried to set it, using the wrong buffer type:
      Glib::RefPtr<gtksourceview::SourceBuffer> buffer = gtksourceview::SourceBuffer::create(language);
-     buffer->set_highlight();
+     buffer->set_highlight_syntax();
      m_text_view->set_buffer(buffer);
   }
 
@@ -70,7 +73,8 @@ void Dialog_ButtonScript::set_script(const sharedptr<const LayoutItem_Button>& s
   m_table_name = table_name;  //Used for lookup combo boxes.
 
   m_text_view->get_buffer()->set_text( script->get_script() );
-
+	
+  m_entry_title->set_text (script->get_title());
   //set_blocked(false);
 
   //Dialog_Properties::set_modified(false);
@@ -80,11 +84,17 @@ sharedptr<LayoutItem_Button> Dialog_ButtonScript::get_script() const
 {
   sharedptr<LayoutItem_Button> result = glom_sharedptr_clone(m_script); //Start with the old details, to preserve anything that is not in our UI.
 
-  result->set_script( m_text_view->get_buffer()->get_text() );
+  get_script (result);
 
   return result;
 }
 
+void Dialog_ButtonScript::get_script (const sharedptr<LayoutItem_Button>& script) const
+{
+  script->set_script (m_text_view->get_buffer()->get_text() );
+  script->set_title (m_entry_title->get_text());
+}
+
 void Dialog_ButtonScript::on_button_test()
 {
   const Glib::ustring calculation = m_text_view->get_buffer()->get_text();
