@@ -58,13 +58,14 @@ protected:
   sharedptr<LayoutItem_Button> offer_button_script_edit(const sharedptr<const LayoutItem_Button>& button);
 
   //signal handlers:
-  void on_button_field_up();
-  void on_button_field_down();
+  void on_button_up();
+  void on_button_down();
   void on_button_field_delete();
-  virtual void on_button_field_add(); //overridden in derived class. 
-  void on_button_field_add_group();
+  virtual void on_button_add_field(); //overridden in derived class. 
+  void on_button_add_group();
   void on_button_add_notebook();
   void on_button_add_related();
+  void on_button_add_related_calendar();
   void on_button_add_button();
   void on_button_add_text();
   void on_button_add_image();
@@ -74,28 +75,33 @@ protected:
 
   void on_cell_data_name(Gtk::CellRenderer* renderer, const Gtk::TreeModel::iterator& iter);
   void on_cell_data_title(Gtk::CellRenderer* renderer, const Gtk::TreeModel::iterator& iter);
-  void on_cell_data_columns_count(Gtk::CellRenderer* renderer, const Gtk::TreeModel::iterator& iter);
+  void on_cell_data_group_columns(Gtk::CellRenderer* renderer, const Gtk::TreeModel::iterator& iter);
+  void on_cell_data_column_width(Gtk::CellRenderer* renderer, const Gtk::TreeModel::iterator& iter);
 
   void on_treeview_cell_edited_name(const Glib::ustring& path_string, const Glib::ustring& new_text);
   void on_treeview_cell_edited_title(const Glib::ustring& path_string, const Glib::ustring& new_text);
-  void on_treeview_cell_edited_columns_count(const Glib::ustring& path_string, const Glib::ustring& new_text);
+  void on_treeview_cell_edited_group_columns(const Glib::ustring& path_string, const Glib::ustring& new_text);
+  void on_treeview_cell_edited_column_width(const Glib::ustring& path_string, const Glib::ustring& new_text);
 
   Gtk::TreeModel::iterator append_appropriate_row();
 
   Gtk::TreeView* m_treeview_fields;
   Gtk::TreeView::Column* m_treeview_column_title;
+  Gtk::TreeView::Column* m_treeview_column_group_columns;
+  Gtk::TreeView::Column* m_treeview_column_column_width;
 
   // Only one of these boxes should be shown:
   Gtk::Box* m_box_table_widgets;
   Gtk::Box* m_box_related_table_widgets;
   Gtk::Frame* m_box_related_navigation;
 
-  Gtk::Button* m_button_field_up;
-  Gtk::Button* m_button_field_down;
-  Gtk::Button* m_button_field_add;
-  Gtk::Button* m_button_field_add_group;
+  Gtk::Button* m_button_up;
+  Gtk::Button* m_button_down;
+  Gtk::Button* m_button_add_field;
+  Gtk::Button* m_button_add_group;
   Gtk::Button* m_button_add_notebook;
   Gtk::Button* m_button_add_related;
+  Gtk::Button* m_button_add_related_calendar;
   Gtk::Button* m_button_add_button;
   Gtk::Button* m_button_add_text;
   Gtk::Button* m_button_add_image;
