@@ -22,10 +22,10 @@
 #define BOX_DB_TABLE_RELATIONSHIPS_H
 
 #include "../box_db_table.h"
+#include <glom/utility_widgets/adddel/adddel_withbuttons.h>
 
-/**
-  *@author Murray Cumming
-  */
+namespace Glom
+{
 
 class Box_DB_Table_Relationships : public Box_DB_Table
 {
@@ -42,10 +42,10 @@ protected:
   virtual bool fill_from_database();
 
   //Signal handlers:
-  virtual void on_adddel_user_activated(const Gtk::TreeModel::iterator& row, guint col);
-  virtual void on_adddel_user_changed(const Gtk::TreeModel::iterator& row, guint col);
-  virtual void on_adddel_user_requested_delete(const Gtk::TreeModel::iterator& rowStart, const Gtk::TreeModel::iterator& rowEnd);
-  virtual void on_adddel_user_added(const Gtk::TreeModel::iterator& row);
+  void on_adddel_user_activated(const Gtk::TreeModel::iterator& row, guint col);
+  void on_adddel_user_changed(const Gtk::TreeModel::iterator& row, guint col);
+  void on_adddel_user_requested_delete(const Gtk::TreeModel::iterator& rowStart, const Gtk::TreeModel::iterator& rowEnd);
+  void on_adddel_user_added(const Gtk::TreeModel::iterator& row);
 
   guint m_colName, m_colTitle, m_colFromField, m_colToTable, m_colToField, m_colAllowEdit, m_colAutoCreate;
 
@@ -53,5 +53,7 @@ protected:
   Gtk::Button m_Button_Guess;
 };
 
+} //namespace Glom
+
 #endif //BOX_DB_TABLE_RELATIONSHIPS_H
 
