@@ -24,19 +24,24 @@
 #include "dialog_design.h"
 #include "box_db_table_relationships.h"
 
+namespace Glom
+{
+
 class Dialog_Relationships : public Dialog_Design
 {
 public:
   Dialog_Relationships(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade);
   virtual ~Dialog_Relationships();
 
-  virtual void init_db_details(const Glib::ustring& strDatabaseName, const Glib::ustring& strTableName);
+  virtual bool init_db_details(const Glib::ustring& table_name);
 
   virtual void on_hide();
-  
+
 protected:
 
   Box_DB_Table_Relationships* m_box;
 };
 
+} //namespace Glom
+
 #endif //DIALOG_RELATIONSHIPS_H
