@@ -19,26 +19,356 @@
  */
  
 #include "flowtable.h"
-#include "gtk/gtkwidget.h"
-#include "gdk/gdktypes.h"
+#include "layoutwidgetbase.h"
+#include <gtk/gtkwidget.h>
+#include <gdk/gdktypes.h>
+#include <iostream>
+#include <gdkmm/window.h>
+#include <gtk/gtk.h>
 
-FlowTable::FlowTableItem::FlowTableItem()
-: m_expand_first_full(false),
-  m_expand_second(false)
+// So we don't need to check for the condition above all the time
+
+// A GObject for the flowtable to allow overriding vfuncs even with the
+// reduced glibmm API
+namespace
 {
+#if 0
+#define GLOM_TYPE_FLOWTABLE (glom_flowtable_get_type())
+#define GLOM_FLOWTABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj), GLOM_TYPE_FLOWTABLE, GlomFlowtable))
+#endif
+
+#ifndef GLIBMM_VFUNCS_ENABLED
+  GtkContainerClass* parent_class = NULL;
+#endif
+
+#if 0
+  struct GlomFlowtableClass
+  {
+    GtkContainerClass parent_class;
+  }
+
+  struct GlomFlowtable
+  {
+    GtkContainer parent_instance;
+    Glom::FlowTable* cpp_table;
+  }
+
+  void glom_flowtable_init(GlowFlowtable* object)
+  {
+    object->cpp_table = NULL;
+  }
+#endif
+
+  void register_vfuncs(Glom::FlowTable& table)
+  {
+  }
+#if 0
+  void glom_flowtable_class_init(gpointer g_class, gpointer class_data)
+  {
+    GtkWidgetClass* widget_class = GTK_WIDGET_CLASS(g_class);
+    GtkContainerClass* container_class = GTK_CONTAINER_CLASS(g_class);
+
+    parent_class = GTK_CONTAINER_CLASS(g_type_class_peek_parent(g_class));
+
+  }
+
+  GType glom_flowtable_get_type()
+  {
+    static GType flowtable_type = 0;
+
+    if(!flowtable_type)
+    {
+      static const GTypeInfo flowtable_type_info = {
+        sizeof(GlomFlowtableClass),  /* class_size */
+        NULL,                        /* base_init */
+        NULL,                        /* base_finalize */
+        glom_flowtable_class_init,   /* class_init */
+        NULL,                        /* class_finalize */
+        NULL,                        /* class_data */
+        sizeof(GlomFlowtable),       /* instance_size */
+        0,                           /* n_preallocs */
+        glom_flowtable_init,         /* instance_init */
+        NULL                         /* value_table */
+      };
+
+      flowtable_type = g_type_register_static(GTK_TYPE_CONTAINER, "GlomFlowtable", &flowtable_type_info, 0);
+    }
+
+    return flowtable_type;
+  }
+#endif
 }
 
+namespace Glom
+{
+#ifndef GLIBMM_VFUNCS_ENABLED
+  void FlowTable::glom_forall_impl(GtkContainer* container, gboolean include_internals, GtkCallback callback, gpointer callback_data)
+  {
+    Glib::ObjectBase *const obj_base = static_cast<Glib::ObjectBase*>(
+      Glib::ObjectBase::_get_current_wrapper((GObject*)container));
+
+    if(obj_base)
+    {
+      Glom::FlowTable* table = dynamic_cast<Glom::FlowTable*>(obj_base);
+      g_assert(table);
+      table->forall_vfunc(include_internals, callback, callback_data);
+    }
+    else if(parent_class->forall)
+      parent_class->forall(container, include_internals, callback, callback_data);
+  }
+
+  GType FlowTable::glom_child_type_impl(GtkContainer* container)
+  {
+    Glib::ObjectBase *const obj_base = static_cast<Glib::ObjectBase*>(
+      Glib::ObjectBase::_get_current_wrapper((GObject*)container));
+
+    if(obj_base)
+    {
+      Glom::FlowTable* table = dynamic_cast<Glom::FlowTable*>(obj_base);
+      g_assert(table);
+      return table->child_type_vfunc();
+    }
+    else if(parent_class->child_type)
+      return parent_class->child_type(container);
+    else
+      return G_TYPE_NONE;
+  }
+#endif // !GLIBMM_VFUNCS_ENABLED
+
+#ifndef GLIBMM_DEFAULT_SIGNAL_HANDLERS_ENABLED
+  // TODO: It is probably OK doing static_cast here.
+  void FlowTable::glom_size_request_impl(GtkWidget* widget, GtkRequisition* requisition)
+  {
+    Glib::ObjectBase *const obj_base = static_cast<Glib::ObjectBase*>(
+      Glib::ObjectBase::_get_current_wrapper((GObject*)widget));
+
+    if(obj_base)
+    {
+      Glom::FlowTable* table = dynamic_cast<Glom::FlowTable*>(obj_base);
+      g_assert(table);
+      table->on_size_request(requisition);
+    }
+    else if(GTK_WIDGET_CLASS(parent_class)->size_request)
+      GTK_WIDGET_CLASS(parent_class)->size_request(widget, requisition);
+  }
+
+  void FlowTable::glom_size_allocate_impl(GtkWidget* widget, GtkAllocation* allocation)
+  {
+    Glib::ObjectBase *const obj_base = static_cast<Glib::ObjectBase*>(
+      Glib::ObjectBase::_get_current_wrapper((GObject*)widget));
+
+    if(obj_base)
+    {
+      Glom::FlowTable* table = dynamic_cast<Glom::FlowTable*>(obj_base);
+      g_assert(table);
+
+      Gtk::Allocation cpp_allocation = Glib::wrap(allocation);
+      table->on_size_allocate(cpp_allocation);
+      *allocation = *cpp_allocation.gobj();
+    }
+    else if(GTK_WIDGET_CLASS(parent_class)->size_allocate)
+      GTK_WIDGET_CLASS(parent_class)->size_allocate(widget, allocation);
+  }
+
+  void FlowTable::glom_add_impl(GtkContainer* container, GtkWidget* widget)
+  {
+    Glib::ObjectBase *const obj_base = static_cast<Glib::ObjectBase*>(
+      Glib::ObjectBase::_get_current_wrapper((GObject*)container));
+
+    if(obj_base)
+    {
+      Glom::FlowTable* table = dynamic_cast<Glom::FlowTable*>(obj_base);
+      g_assert(table);
+      table->on_add(Glib::wrap(widget));
+    }
+    else if(parent_class->add)
+      parent_class->add(container, widget);
+  }
+
+  void FlowTable::glom_remove_impl(GtkContainer* container, GtkWidget* widget)
+  {
+    Glib::ObjectBase *const obj_base = static_cast<Glib::ObjectBase*>(
+      Glib::ObjectBase::_get_current_wrapper((GObject*)container));
+
+    if(obj_base)
+    {
+      Glom::FlowTable* table = dynamic_cast<Glom::FlowTable*>(obj_base);
+      g_assert(table);
+      table->on_remove(Glib::wrap(widget));
+    }
+    else if(parent_class->remove)
+      parent_class->remove(container, widget);
+  }
+
+  void FlowTable::glom_realize_impl(GtkWidget* widget)
+  {
+    Glib::ObjectBase *const obj_base = static_cast<Glib::ObjectBase*>(
+      Glib::ObjectBase::_get_current_wrapper((GObject*)widget));
+
+    if(obj_base)
+    {
+      Glom::FlowTable* table = dynamic_cast<Glom::FlowTable*>(obj_base);
+      g_assert(table);
+      table->on_realize();
+    }
+    else if(GTK_WIDGET_CLASS(parent_class)->realize)
+      GTK_WIDGET_CLASS(parent_class)->realize(widget);
+  }
+
+  void FlowTable::glom_unrealize_impl(GtkWidget* widget)
+  {
+    Glib::ObjectBase *const obj_base = static_cast<Glib::ObjectBase*>(
+      Glib::ObjectBase::_get_current_wrapper((GObject*)widget));
+
+    if(obj_base)
+    {
+      Glom::FlowTable* table = dynamic_cast<Glom::FlowTable*>(obj_base);
+      g_assert(table);
+      table->on_unrealize();
+    }
+    else if(GTK_WIDGET_CLASS(parent_class)->unrealize)
+      GTK_WIDGET_CLASS(parent_class)->unrealize(widget);
+  }
+
+  gboolean FlowTable::glom_expose_event_impl(GtkWidget* widget, GdkEventExpose* event)
+  {
+    Glib::ObjectBase *const obj_base = static_cast<Glib::ObjectBase*>(
+      Glib::ObjectBase::_get_current_wrapper((GObject*)widget));
+
+    if(obj_base)
+    {
+      Glom::FlowTable* table = dynamic_cast<Glom::FlowTable*>(obj_base);
+      g_assert(table);
+      return table->on_expose_event(event);
+    }
+    else if(GTK_WIDGET_CLASS(parent_class)->expose_event)
+      return GTK_WIDGET_CLASS(parent_class)->expose_event(widget, event);
+    else
+      return FALSE;
+  }
+#endif // !GLIBMM_DEFAULT_SIGNAL_HANDLERS_ENABLED
+}
+
+namespace Glom
+{
+
+static void container_forall_callback(GtkWidget* widget_gobj, void* data)
+{
+  #ifdef GLIBMM_EXCEPTIONS_ENABLED
+  try
+  {
+  #endif //GLIBMM_EXCEPTIONS_ENABLED
+    FlowTable::ForallSlot& slot = *static_cast<FlowTable::ForallSlot*>(data);
+    Gtk::Widget *const widget = Glib::wrap(widget_gobj);
+
+    g_return_if_fail(widget != 0);
+
+    slot(*widget);
+  #ifdef GLIBMM_EXCEPTIONS_ENABLED
+  }
+  catch(...)
+  {
+    Glib::exception_handlers_invoke();
+  }
+  #endif //GLIBMM_EXCEPTIONS_ENABLED
+}
+
+  
+FlowTable::FlowTableItem::FlowTableItem(Gtk::Widget* first, FlowTable* flowtable)
+: m_first(first),
+  m_second(0),
+  m_expand_first_full(false),
+  m_expand_second(false)
+{
+
+}
+
+FlowTable::FlowTableItem::FlowTableItem(Gtk::Widget* first, Gtk::Widget* second, FlowTable* flowtable)
+: m_first (first),
+  m_second(second),
+  m_expand_first_full(false),
+  m_expand_second(false)
+{
+
+}
+
+
 FlowTable::FlowTable()
-: m_columns_count(1),
+:
+#if !defined(GLIBMM_VFUNCS_ENABLED) || !defined(GLIBMM_DEFAULT_SIGNAL_HANDLERS_ENABLED)
+  // This creates a custom GType for us, to override vfuncs and default
+  // signal handlers even with the reduced API.
+  // TODO: It is necessary to do this in all derived classes which is
+  // rather annoying, though I don't see another possibility at the moment. armin.
+  Glib::ObjectBase("Glom_FlowTable"),
+#endif // !defined(GLIBMM_VFUNCS_ENABLED) || !defined(GLIBMM_DEFAULT_SIGNAL_HANDLERS_ENABLED)
+  m_columns_count(1),
   m_padding(0),
   m_design_mode(false)
 {
+#if !defined(GLIBMM_VFUNCS_ENABLED) || !defined(GLIBMM_DEFAULT_SIGNAL_HANDLERS_ENABLED)
+  // TODO: Thread safety?
+  // TODO: We could also set up a Glib::Class derived object with a custom
+  // class init function so we do not need to lookup the class object by
+  // the underlaying gobj() as we do currently.
+  if(!parent_class)
+  {
+    GtkContainerClass* container_class = G_TYPE_INSTANCE_GET_CLASS(gobj(), G_TYPE_FROM_INSTANCE(gobj()), GtkContainerClass);
+    GtkWidgetClass* widget_class = GTK_WIDGET_CLASS(container_class);
+
+#ifndef GLIBMM_VFUNCS_ENABLED
+    container_class->forall = &Glom::FlowTable::glom_forall_impl;
+    container_class->child_type = &Glom::FlowTable::glom_child_type_impl;
+#endif // !GLIBMM_VFUNCS_ENABLED
+
+#ifndef GLIBMM_DEFAULT_SIGNAL_HANDLERS_ENABLED
+    container_class->add = &Glom::FlowTable::glom_add_impl;
+    container_class->remove = &Glom::FlowTable::glom_remove_impl;
+
+    widget_class->size_request = &Glom::FlowTable::glom_size_request_impl;
+    widget_class->size_allocate = &Glom::FlowTable::glom_size_allocate_impl;
+    widget_class->realize = &Glom::FlowTable::glom_realize_impl;
+    widget_class->unrealize = &Glom::FlowTable::glom_unrealize_impl;
+    widget_class->expose_event = &Glom::FlowTable::glom_expose_event_impl;
+#endif // !GLIBMM_DEFAULT_SIGNAL_HANDLERS_ENABLED
+
+    parent_class = GTK_CONTAINER_CLASS(g_type_class_peek_parent(container_class));
+  }
+#endif // !defined(GLIBMM_VFUNCS_ENABLED) || !defined(GLIBMM_DEFAULT_SIGNAL_HANDLERS_ENABLED)
+
   set_flags(Gtk::NO_WINDOW);
   set_redraw_on_allocate(false);
 }
 
 FlowTable::~FlowTable()
 {
+  //Delete managed children.
+  //(For some reason this is not always happening via Gtk::Container:
+  /* Actualy, don't do this because we would then be double-deleting what the 
+   * Container base class already deleted. We'll have to really find out if/why some 
+   * things are not being deleted. murrayc.
+  bool one_deleted = true;
+  while(!m_children.empty() && one_deleted)
+  {
+    one_deleted = false;
+
+    type_vecChildren::iterator iter = m_children.begin();
+    FlowTableItem& item = *iter;
+
+    //Delete the widgets:
+    if(item.m_first || item.m_second)
+    {
+      if(item.m_first && item.m_first->is_managed_())
+        delete item.m_first;
+
+      if(item.m_second && item.m_second->is_managed_())
+        delete item.m_second;
+
+      m_children.erase(iter);
+      one_deleted = true; //Make sure that we loop again.
+    }
+  }
+  */
 }
 
 void FlowTable::set_design_mode(bool value)
@@ -50,25 +380,84 @@ void FlowTable::set_design_mode(bool value)
 
 void FlowTable::add(Gtk::Widget& first, Gtk::Widget& second, bool expand_second)
 {
-  FlowTableItem item;
-  item.m_first = &first;
-  item.m_second = &second;
+  FlowTableItem item (&first, &second, this);
+  
   item.m_expand_second = expand_second; //Expand to fill the width for all of the second item.
   m_children.push_back(item);
-
-  gtk_widget_set_parent(first.gobj(), GTK_WIDGET(gobj()));
-  gtk_widget_set_parent(second.gobj(), GTK_WIDGET(gobj()));
+  gtk_widget_set_parent(GTK_WIDGET (item.m_first->gobj()), GTK_WIDGET(gobj()));
+  gtk_widget_set_parent(GTK_WIDGET (item.m_second->gobj()), GTK_WIDGET(gobj()));
 }
 
 void FlowTable::add(Gtk::Widget& first, bool expand)
 {
-  FlowTableItem item;
-  item.m_first = &first;
+  FlowTableItem item(&first, this);
   item.m_expand_first_full = expand; //Expand to fill the width for first and second.
-  item.m_second = 0;
   m_children.push_back(item);
+  gtk_widget_set_parent(GTK_WIDGET (item.m_first->gobj()), GTK_WIDGET(gobj()));
+}
 
-  gtk_widget_set_parent(first.gobj(), GTK_WIDGET(gobj()));
+void FlowTable::insert_before(Gtk::Widget& first, Gtk::Widget& before, bool expand)
+{
+  FlowTableItem item(&first, this);
+  item.m_expand_first_full = expand;
+  insert_before (item, before);
+}
+
+void FlowTable::insert_before(Gtk::Widget& first, Gtk::Widget& second, Gtk::Widget& before, bool expand_second)
+{
+  FlowTableItem item(&first, &second, this);
+  item.m_expand_second = expand_second;
+  insert_before (item, before);
+}
+
+void FlowTable::insert_before(FlowTableItem& item, Gtk::Widget& before)
+{
+  bool found = false;
+  std::vector<FlowTableItem>::iterator pos;
+  for (pos = m_children.begin(); pos != m_children.end(); pos++)
+  {
+    FlowTableItem* item = &(*pos);
+    if (item->m_first)
+    {
+      if (item->m_first->gobj() == before.gobj())
+      {
+        found = true;
+        break;
+      }
+      Gtk::Alignment* alignment = dynamic_cast<Gtk::Alignment*>(item->m_first);
+      if (alignment && alignment->get_child()->gobj() == before.gobj())
+      {
+        found = true;
+        break;
+      }
+    }
+    if (item->m_second)
+    {
+      if (item->m_second->gobj() == before.gobj())
+      {
+        found = true;
+        break;
+      }
+      Gtk::Alignment* alignment = dynamic_cast<Gtk::Alignment*>(item->m_second);
+      if (alignment && alignment->get_child()->gobj() == before.gobj())
+      {
+        found = true;
+        break;
+      }
+    }    
+  }
+ 
+  gtk_widget_set_parent(GTK_WIDGET (item.m_first->gobj()), GTK_WIDGET(gobj()));
+  if (item.m_second)
+  {
+    gtk_widget_set_parent(GTK_WIDGET (item.m_second->gobj()), GTK_WIDGET(gobj()));
+  }
+  if (pos == m_children.end())
+  {
+    m_children.push_back(item);
+  } else {
+    m_children.insert(pos, item);
+  }
 }
 
 void FlowTable::set_columns_count(guint value)
@@ -78,6 +467,7 @@ void FlowTable::set_columns_count(guint value)
     value = 1;
 
   m_columns_count = value;
+  //std::cout << "FlowTable::set_columns_count(): m_columns_count=:" << m_columns_count << std::endl;
 }
 
 void FlowTable::get_item_requested_width(const FlowTableItem& item, int& first, int& second) const
@@ -95,7 +485,7 @@ void FlowTable::get_item_requested_width(const FlowTableItem& item, int& first,
     {
       //Discover how much space this child needs:
       Gtk::Requisition child_requisition_request;
-      first_widget->size_request(child_requisition_request);
+      child_requisition_request = first_widget->size_request();
       first = child_requisition_request.width;
     }
 
@@ -103,7 +493,7 @@ void FlowTable::get_item_requested_width(const FlowTableItem& item, int& first,
     {
       //Discover how much space this child needs:
       Gtk::Requisition child_requisition_request;
-      second_widget->size_request(child_requisition_request);
+      child_requisition_request = second_widget->size_request();
       second = child_requisition_request.width;
     }
   }
@@ -123,7 +513,7 @@ int FlowTable::get_item_requested_height(const FlowTableItem& item) const
     {
       // Ask the child how much space it needs:
       Gtk::Requisition child_requisition;
-      first->size_request(child_requisition);
+      child_requisition = first->size_request();
 
       max_child_height = child_requisition.height;
     }
@@ -132,7 +522,7 @@ int FlowTable::get_item_requested_height(const FlowTableItem& item) const
     {
       // Ask the child how much space it needs:
       Gtk::Requisition child_requisition;
-      second->size_request(child_requisition);
+      child_requisition = second->size_request();
 
       max_child_height = MAX(max_child_height, child_requisition.height);
     }
@@ -157,7 +547,7 @@ int FlowTable::get_column_height(guint start_widget, guint widget_count, int& to
   guint i = 0;
   for(i = start_widget; i < (start_widget+widget_count);  ++i)
   {
-    FlowTableItem item = m_children[i];
+    const FlowTableItem& item = m_children[i];
     int item_height = get_item_requested_height(item);
 
     int item_width_first = 0;
@@ -269,6 +659,11 @@ int FlowTable::get_minimum_column_height(guint start_widget, guint columns_count
 
 void FlowTable::on_size_request(Gtk::Requisition* requisition)
 {
+  // Set a minimum size so people are able to drag items into the
+  // table
+  const int MIN_HEIGHT = (m_design_mode ? 50 : 0);
+  const int MIN_WIDTH = (m_design_mode ? 100 : 0);
+  
   //Initialize the output parameter:
   *requisition = Gtk::Requisition();
 
@@ -277,8 +672,8 @@ void FlowTable::on_size_request(Gtk::Requisition* requisition)
   int total_width = 0;
   const int column_height = get_minimum_column_height(0, m_columns_count, total_width); //This calls itself recursively.
 
-  requisition->height = column_height;
-  requisition->width = total_width;
+  requisition->height = (column_height > MIN_HEIGHT ? column_height : MIN_HEIGHT);
+  requisition->width = (total_width > MIN_WIDTH ? total_width : MIN_WIDTH);
 }
 
 //Give it whatever height/width it wants, at this location:
@@ -306,7 +701,7 @@ Gtk::Allocation FlowTable::assign_child(Gtk::Widget* widget, int x, int y)
 {
   //Discover how much space this child needs:
   Gtk::Requisition child_requisition_request;
-  widget->size_request(child_requisition_request);  //TODO: This method should be const:
+  child_requisition_request = widget->size_request();
 
   //Give it as much space as it wants:
   return assign_child(widget, x, y, child_requisition_request.width, child_requisition_request.height);
@@ -328,7 +723,7 @@ void FlowTable::get_item_max_width_requested(guint start, guint height, guint& f
   guint i = start;
   while( (height_so_far < height) && (i < m_children.size()))
   {
-    FlowTableItem item = m_children[i];
+    const FlowTableItem& item = m_children[i];
     Gtk::Widget* first = item.m_first;
     Gtk::Widget* second = item.m_second;
 
@@ -344,7 +739,7 @@ void FlowTable::get_item_max_width_requested(guint start, guint height, guint& f
     if(child_is_visible(first))
     {
       Gtk::Requisition child_requisition;
-      first->size_request(child_requisition);
+      child_requisition = first->size_request();
 
       if(child_is_visible(second))
       {
@@ -365,7 +760,7 @@ void FlowTable::get_item_max_width_requested(guint start, guint height, guint& f
     if(child_is_visible(second))
     {
       Gtk::Requisition child_requisition;
-      second->size_request(child_requisition);
+      child_requisition = second->size_request();
 
       item_second_width = child_requisition.width;
       //g_warning("item_second_width=%d", item_second_width);
@@ -437,7 +832,7 @@ void FlowTable::on_size_allocate(Gtk::Allocation& allocation)
   {
     FlowTableItem& item = m_children[i];
 
-    int item_height = get_item_requested_height(item);
+    const int item_height = get_item_requested_height(item);
 
     //Start a new column if necessary:
     int bottom = allocation.get_y() + allocation.get_height();   
@@ -554,12 +949,10 @@ GtkType FlowTable::child_type_vfunc() const
 
 void FlowTable::on_add(Gtk::Widget* child)
 {
-  FlowTableItem item;
-  item.m_first = child;
-  item.m_second = 0;
+  FlowTableItem item(child, 0);
   m_children.push_back(item);
   
-  gtk_widget_set_parent(child->gobj(), GTK_WIDGET(gobj()));
+  gtk_widget_set_parent(GTK_WIDGET (item.m_first->gobj()), GTK_WIDGET(gobj()));
 
   //This is protected, but should be public: child.set_parent(*this);
 
@@ -580,7 +973,7 @@ void FlowTable::on_remove(Gtk::Widget* child)
       if(item.m_first == child)
       {
             //g_warning("FlowTable::on_remove unparenting first");
-        gtk_widget_unparent(item.m_first->gobj()); //This is protected, but should be public: child.unparent();
+        gtk_widget_unparent(GTK_WIDGET (item.m_first->gobj())); //This is protected, but should be public: child.unparent();
         item.m_first = 0;
 
        if(visible)
@@ -590,7 +983,7 @@ void FlowTable::on_remove(Gtk::Widget* child)
       if(item.m_second == child)
       {
         //g_warning("FlowTable::on_remove unparenting second");
-        gtk_widget_unparent(item.m_second->gobj()); //This is protected, but should be public: child.unparent();
+        gtk_widget_unparent(GTK_WIDGET (item.m_second->gobj())); //This is protected, but should be public: child.unparent();
         item.m_second = 0;
 
         if(visible)
@@ -619,6 +1012,12 @@ void FlowTable::forall_vfunc(gboolean /* include_internals */, GtkCallback callb
   }
 }
 
+void FlowTable::forall(const ForallSlot& slot)
+{
+  ForallSlot slot_copy (slot);
+  gtk_container_forall(gobj(), &container_forall_callback, &slot_copy);
+}
+
 /** Sets the padding to put between the child widgets.
  */
 void FlowTable::set_padding(guint padding)
@@ -663,7 +1062,7 @@ void FlowTable::remove_all()
       if(widget->is_managed_())
         widget->reference();
 
-      gtk_widget_unparent(widget->gobj());
+      gtk_widget_unparent(GTK_WIDGET (iter->m_first->gobj()));
     }
 
     if(iter->m_second)
@@ -673,7 +1072,7 @@ void FlowTable::remove_all()
       if(widget->is_managed_())
         widget->reference();
 
-      gtk_widget_unparent(widget->gobj());
+      gtk_widget_unparent(GTK_WIDGET (iter->m_second->gobj()));
     }
 
   }
@@ -683,7 +1082,12 @@ void FlowTable::remove_all()
 
 void FlowTable::on_realize()
 {
+#ifdef GLIBMM_DEFAULT_SIGNAL_HANDLERS_ENABLED
   Gtk::Container::on_realize();
+#else
+  if(GTK_WIDGET_CLASS(parent_class)->realize)
+    GTK_WIDGET_CLASS(parent_class)->realize(GTK_WIDGET(gobj()));
+#endif
 
   if(!m_refGdkWindow)
   {
@@ -697,7 +1101,12 @@ void FlowTable::on_unrealize()
   m_refGdkWindow.clear();
   m_refGC.clear();
 
+#ifdef GLIBMM_DEFAULT_SIGNAL_HANDLERS_ENABLED
   Gtk::Container::on_unrealize();
+#else
+  if(GTK_WIDGET_CLASS(parent_class)->unrealize)
+    GTK_WIDGET_CLASS(parent_class)->unrealize(GTK_WIDGET(gobj()));
+#endif
 }
 
 bool FlowTable::on_expose_event(GdkEventExpose* event)
@@ -710,16 +1119,6 @@ bool FlowTable::on_expose_event(GdkEventExpose* event)
       m_refGC = Gdk::GC::create(m_refGdkWindow);
       m_refGC->set_line_attributes(1 /* width */, Gdk::LINE_ON_OFF_DASH, Gdk::CAP_NOT_LAST, Gdk::JOIN_MITER);
 
-      //Glib::RefPtr<Gdk::Colormap> colormap = get_default_colormap();
-      //Gdk::Color color_blue("blue");
-      //Gdk::Color color_red("red");
-      //colormap->alloc_color(color_blue);
-      //colormap->alloc_color(color_red);
-
-      //m_refGdkWindow->set_background( color_red );
-      //m_refGdkWindow->clear();
-      //m_refGC->set_foreground( color_blue );
-
       for(type_vecLines::iterator iter = m_lines_horizontal.begin(); iter != m_lines_horizontal.end(); ++iter)
       {
         //TODO: Add draw_line(point, point) to gdkmm:
@@ -732,9 +1131,15 @@ bool FlowTable::on_expose_event(GdkEventExpose* event)
       }
     }
   }
-
+#ifdef GLIBMM_DEFAULT_SIGNAL_HANDLERS_ENABLED
   return Gtk::Container::on_expose_event(event);
+#else
+  if(GTK_WIDGET_CLASS(parent_class)->expose_event)
+    return GTK_WIDGET_CLASS(parent_class)->expose_event(GTK_WIDGET(gobj()), event);
+  return true;
+#endif
 }
 
+} //namespace Glom
 
 
